/**********************************************************************
 *
 * Filename:    timer.c
 * 
 * Description: Timer device driver.
 *
 * Notes:       This file is specific to the Arcom board.
 * 
 * Copyright (c) 2006 Anthony Massa and Michael Barr. All rights reserved.
 * This code is from the book Programming Embedded Systems, With C and
 * GNU Development Tools, 2nd Edition.
 * It is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://www.oreilly.com/catalog/embsys2.
 *
 **********************************************************************/

/**********************************************************************
 *
 * Notes:       This file is specific to the LDS4000 board
 * 
 * Copyright (c) 2007 Corebell System, Inc. All rights reserved.
 * You may study, use, and modify it for any non-commercial purpose,
 * including teaching and use in open-source projects.
 * You may distribute it non-commercially as long as you retain this notice.
 * For a commercial use license, or to purchase the book,
 * please visit http://corebell.co.kr
 *
 **********************************************************************/
 
#include <cyg/kernel/kapi.h>
#include <cyg/hal/hal_intr.h>
#include "stdint.h"
#include "pxa270.h"
#include "lds4000.h"
#include "led.h"
#include "timer.h"


#define TIMER_INTERVAL_500MS        (0x001C2000)


/* Declare the ISR variables. */
cyg_handle_t timerInterruptHdl;
cyg_interrupt timerInterruptObj;
cyg_vector_t timerInterruptVector = TIMER1_INT;

cyg_sem_t ledToggleSemaphore;


/**********************************************************************
 *
 * Function:    timerInit
 *
 * Description: Initialize and start the timer.
 * 
 * Notes:       This function is specific to the Arcom board.
 *              Ensure an ISR has been installed for the timer prior
 *              to calling this routine.
 *
 * Returns:     None.
 *
 **********************************************************************/
void timerInit(void)
{
    static int bInitialized = FALSE;

    /* Initialize the timer only once. */
    if (bInitialized == FALSE)
    {
        /* Acknowledge any outstanding timer interrupts. */
        TIMER_STATUS_REG = TIMER_1_MATCH;

        /* Initialize the timer interval. */
        TIMER_1_MATCH_REG = (TIMER_COUNT_REG + TIMER_INTERVAL_500MS);

        /* Enable the timer interrupt in the timer peripheral. */
        TIMER_INT_ENABLE_REG |= TIMER_1_INTEN;

        /* Enable the timer interrupt in the interrupt controller. */
        INTERRUPT_ENABLE_REG |= TIMER_1_ENABLE;

        bInitialized = TRUE;
    }
}


/**********************************************************************
 *
 * Function:    timerIsr
 *
 * Description: Interrupt service routine for the timer interrupt.
 * 
 * Notes:       
 *
 * Returns:     Bitmask to inform operating system that the
 *              interrupt has been handled and to schedule the
 *              deferred service routine.
 *
 **********************************************************************/
uint32_t timerIsr(cyg_vector_t vector, cyg_addrword_t data)
{
    /* Block the timer interrupt from occurring until the DSR runs. */
    cyg_interrupt_mask(timerInterruptVector);

    /* Acknowledge the interrupt in the interrupt controller and the
     * timer peripheral. */
    cyg_interrupt_acknowledge(timerInterruptVector);
    TIMER_STATUS_REG = TIMER_1_MATCH;

    /* Inform the operating system that the interrupt is handled by this
     * ISR and that the DSR needs to run. */
    return (CYG_ISR_HANDLED | CYG_ISR_CALL_DSR);
}


/**********************************************************************
 *
 * Function:    timerDsr
 *
 * Description: Deferred service routine for the timer interrupt.
 * 
 * Notes:       
 *
 * Returns:     None.
 *
 **********************************************************************/
void timerDsr(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
    /* Signal the task to toggle the LED. */
    cyg_semaphore_post(&ledToggleSemaphore);

    /* Set the new timer interval. */
    TIMER_1_MATCH_REG = (TIMER_COUNT_REG + TIMER_INTERVAL_500MS);

    /* Enable processing of incoming timer interrupts. */
    cyg_interrupt_unmask(timerInterruptVector);
}

